import react from '@vitejs/plugin-react';
import dts from 'vite-plugin-dts';
import { createRequire } from 'module';
const require = createRequire(import.meta.url);

export const ESM模块BABEL编译配置 = {
  presets: [
    [
      require.resolve('@babel/preset-env'),
      {
        // https://babel.dev/docs/en/babel-preset-env#modules
        modules: false,
      },
    ],
    require.resolve('@babel/preset-react'),
    require.resolve('@babel/preset-typescript'),
  ],
  plugins: [
    [
      require.resolve('@babel/plugin-transform-runtime'),
      { useESModules: true },
    ],
    [require.resolve('@babel/plugin-proposal-decorators'), { legacy: true }],
    [require.resolve('@babel/plugin-proposal-class-properties')],
    require.resolve('@babel/plugin-proposal-optional-chaining'),
  ],
};

export const UMD模块BABEL编译配置 = {
  presets: [
    require.resolve('@babel/preset-env'),
    require.resolve('@babel/preset-react'),
    require.resolve('@babel/preset-typescript'),
  ],
  plugins: [
    require.resolve('@babel/plugin-transform-runtime'),
    [require.resolve('@babel/plugin-proposal-decorators'), { legacy: true }],
    [require.resolve('@babel/plugin-proposal-class-properties')],
    require.resolve('@babel/plugin-proposal-optional-chaining'),
  ],
};

const rollupOptions = {
  external: ['react', 'react-dom', 'moment'],
  output: {
    extend: true,
    globals: {
      react: 'React',
      moment: 'moment',
      'react-dom': 'ReactDOM',
    },
  },
  plugins: [],
};

export const vite构建配置 = {
  build: {
    rollupOptions,
    minify: false,
    outDir: 'dist',
    lib: {
      entry: './src/index.ts',
      name: 'KOOK-UI',
      formats: ['umd'],
    },
  },
  plugins: [react(), dts()],
  resolve: {},
};
