import gulp from 'gulp';
import babel from 'gulp-babel';
import through2 from 'through2';
import path from 'path';
import less from 'gulp-less';
import { build as viteBuild } from 'vite';
import {
  ESM模块BABEL编译配置,
  UMD模块BABEL编译配置,
  vite构建配置,
} from './configs.mjs';

const 获取项目路径 = (dir = './') => {
  return path.join(process.cwd(), dir);
};

const paths = {
  scripts: (propsPath) => [
    获取项目路径(`${propsPath}/**/*.{ts,tsx,js,jsx}`),
    获取项目路径(`!${propsPath}/**/__tests__/*.{ts,tsx,js,jsx}`),
  ],
  styles: (path) => 获取项目路径(`${path}/**/*.less`),
};

const 剔除样式导入代码 = (文件内容) => {
  return 文件内容.replace(`import './styles/index.less';`, '');
};

function 编译脚本函数({ 入口, 模式, 出口 }) {
  const { scripts } = paths;

  const gres = gulp
    .src(scripts(入口))
    // 使用gulp-babel处理
    .pipe(babel(模式 === 'es' ? ESM模块BABEL编译配置 : UMD模块BABEL编译配置))
    .pipe(
      through2.obj(function z(file, encoding, next) {
        if (file.path.match(/(\/|\\)index\.js/)) {
          const 文件内容 = file.contents.toString(encoding);
          file.contents = Buffer.from(剔除样式导入代码(文件内容)); // 处理文件内容
        }
        this.push(file.clone());

        next();
      }),
    )
    .pipe(gulp.dest(出口));
  return gres;
}

export const 打包为ESM模块 = (配置) => {
  gulp.task('compileESM', () => {
    return 编译脚本函数(配置);
  });

  return new Promise((res) => {
    return gulp.series('compileESM', () => {
      res(true);
    })();
  });
};

export const 打包为UMD模块 = (配置) => {
  gulp.task('compileUMD', () => {
    return 编译脚本函数(配置);
  });

  return new Promise((res) => {
    return gulp.series('compileUMD', () => {
      res(true);
    })();
  });
};

export const 复制less = ({ 入口, 出口 }) => {
  gulp.task('less2css', () => {
    const source = gulp.src(paths.styles(入口));
    source.pipe(gulp.dest(出口));
    return source;
  });
  return new Promise((res) => {
    return gulp.series('less2css', () => {
      res(true);
    })();
  });
};

export const 打包样式和类型声明文件 = async () => {
  await viteBuild(vite构建配置);
};
