import fs from 'fs-extra';
import path from 'path';
import fetch from 'node-fetch';
import csv from 'csvtojson';
import prettier from 'prettier';
import compressing from 'compressing';
import { get } from 'lodash-es';

const getHeaders = (token) => {
  return {
    'Content-Type': 'application/json',
    Authorization: `Bearer ${token}`,
  };
};

const 公共接口前缀 = (项目id) =>
  `https://api.crowdin.com/api/v2/projects/${项目id}/translations/builds`;

export const 构建翻译文件获取构建id = async ({ 项目id, token }) => {
  console.log('开始构建翻译');
  const 构建接口响应 = await fetch(公共接口前缀(项目id), {
    method: 'post',
    headers: getHeaders(token),
  });
  const 构建结果数据 = await 构建接口响应.json();
  const 构建id = get(构建结果数据, 'data.id');
  console.log('构建ID：', 构建id);
  return 构建id;
};

export const 替换翻译文件 = async () => {
  await compressing.zip.uncompress(
    path.resolve('./i18n.zip'),
    path.resolve('./i18n'),
  );

  console.log('解压i18压缩成功');

  fs.removeSync(path.resolve('./i18n.zip'));

  fs.copyFileSync(
    path.resolve('./i18n/Core/global.csv'),
    path.resolve('./global.csv'),
  );

  fs.removeSync(path.resolve('./i18n'));

  // 清空目录
  fs.emptyDirSync('./en');
  fs.emptyDirSync('./ja');
  fs.emptyDirSync('./zh');
  fs.emptyDirSync('./zh-tw');

  // 中文json
  const zhJSON = {};
  // 台湾繁体中文json
  const zhtwJSON = {};
  // 英文json
  const enJSON = {};
  // 日文json
  const jaJSON = {};

  const filePath = path.resolve('./', 'global.csv');
  const hasCsvFile = fs.existsSync(filePath);
  if (hasCsvFile === false) {
    console.info('当前文件夹内不存在global.csv文件');
    return;
  }

  console.log('开始读取csv文件并翻译为json');

  // 读取csv文件数据
  const csv反序列化数据 = await csv().fromFile('./global.csv');
  csv反序列化数据.forEach((data) => {
    const { key, cn, en, jp, ['zh-tw']: zhTW } = data;
    zhJSON[key] = cn;
    zhtwJSON[key] = zhTW;
    enJSON[key] = en;
    jaJSON[key] = jp;
  });

  // 将翻译文本 写入对应语言的json文件 并 格式化json
  fs.writeFileSync(
    path.resolve('./zh/translation.json'),
    prettier.format(JSON.stringify(zhJSON), { parser: 'json' }),
    { flag: 'w' },
  );
  fs.writeFileSync(
    path.resolve('./zh-tw/translation.json'),
    prettier.format(JSON.stringify(zhtwJSON), { parser: 'json' }),
    { flag: 'w' },
  );
  fs.writeFileSync(
    path.resolve('./en/translation.json'),
    prettier.format(JSON.stringify(enJSON), { parser: 'json' }),
    { flag: 'w' },
  );
  fs.writeFileSync(
    path.resolve('./ja/translation.json'),
    prettier.format(JSON.stringify(jaJSON), { parser: 'json' }),
    { flag: 'w' },
  );

  fs.removeSync(path.resolve('./global.csv'));
  console.log('替换翻译文案结束');
};

export const 查询构建id状态判断是否构建完成 = async ({
  项目id,
  构建id,
  token,
}) => {
  const 构建状态响应 = await fetch(`${公共接口前缀(项目id)}/${构建id}`, {
    method: 'get',
    headers: getHeaders(token),
  });
  const 构建状态结果数据 = await 构建状态响应.json();
  const 状态 = get(构建状态结果数据, 'data.status', '');
  if (状态 === 'finished') {
    return true;
  }
  return false;
};

export const 导出文件链接 = async ({ 项目id, 构建id, token }) => {
  const 导出文件链接响应 = await fetch(
    `${公共接口前缀(项目id)}/${构建id}/download`,
    {
      method: 'get',
      headers: getHeaders(token),
    },
  );
  const 文件链接数据 = await 导出文件链接响应.json();
  const 文件链接 = get(文件链接数据, 'data.url', '');
  return 文件链接;
};

export const 下载文件 = async (文件链接) => {
  console.log('开始下载文件');
  const 文件响应 = await fetch(文件链接, {
    method: 'get',
    headers: {
      'Transfer-Encoding': 'chunked',
    },
  });

  if (await 文件响应.ok) {
    const blob = await 文件响应.blob();
    const arrayBuffer = await blob.arrayBuffer();
    const buffer = Buffer.from(arrayBuffer);
    fs.writeFileSync('./i18n.zip', buffer);
    console.log('文件下载成功');

    替换翻译文件();
  }
};
